@@ -535,6 +535,11 @@ bool DataSource::valid() const
     return false;
 }
 
+bool DataSource::control(NamedList& params)
+{
+    return m_translator && m_translator->control(params);
+}
+
 unsigned long DataSource::Forward(const DataBlock& data, unsigned long tStamp, unsigned long flags)
 {
     Lock mylock(this,100000);
@@ -604,7 +609,7 @@ bool DataSource::attach(DataConsumer* consumer, bool override)
 	    src2->detach(consumer);
 	    s_consSrcMutex.lock();
 	    if (src) {
-		Debug(DebugGoOn,"DataSource %p raced us attaching consumer %p [%p]",
+		Debug(DebugCrit,"DataSource %p raced us attaching consumer %p [%p]",
 		    src,consumer,this);
 		s_consSrcMutex.unlock();
 		consumer->deref();
@@ -976,7 +981,7 @@ void DataEndpoint::setCallRecord(DataConsumer* consumer)
 
 bool DataEndpoint::addSniffer(DataConsumer* sniffer)
 {
-    if (!sniffer)
+    if ((refcount() <= 0) || !sniffer)
 	return false;
     Lock lock(s_dataMutex);
     if (m_sniffers.find(sniffer))
@@ -1008,6 +1013,15 @@ bool DataEndpoint::delSniffer(DataConsumer* sniffer)
     return true;
 }
 
+DataConsumer* DataEndpoint::getSniffer(const String& name, bool ref)
+{
+    if (name.null())
+	return 0;
+    Lock lock(s_dataMutex);
+    DataConsumer* sniffer = static_cast<DataConsumer*>(m_sniffers[name]);
+    return (ref && sniffer && !sniffer->ref()) ? 0 : sniffer;
+}
+
 void DataEndpoint::clearSniffers()
 {
     Lock lock(s_dataMutex);
@@ -1053,10 +1067,12 @@ bool DataEndpoint::clearData(DataNode* node)
 bool DataEndpoint::control(NamedList& params)
 {
     // TODO how do we handle this case????? operation-status
+    DataSource* peerSrc = m_consumer ? m_consumer->getConnSource() : 0;
     return (m_source && m_source->control(params)) ||
 	(m_consumer && m_consumer->control(params)) ||
 	(m_peerRecord && m_peerRecord->control(params)) ||
-	(m_callRecord && m_callRecord->control(params));
+	(m_callRecord && m_callRecord->control(params)) ||
+	(peerSrc && peerSrc->control(params));
 }
 
 
@@ -1578,6 +1594,8 @@ bool DataTranslator::attachChain(DataSource* source, DataConsumer* consumer, boo
 	    DataTranslator* trans = trans2->getFirstTranslator();
 	    trans2->getTransSource()->attach(consumer,override);
 	    source->attach(trans);
+	    trans->attached(true);
+	    trans2->attached(true);
 	    trans->deref();
 	    retv = true;
 	}
@@ -1773,6 +1791,7 @@ DataTranslator* ChainedFactory::create(const DataFormat& sFormat, const DataForm
 	// trans2 may be a chain itself so find the first translator
 	DataTranslator* trans1 = trans2->getFirstTranslator();
 	trans->getTransSource()->attach(trans1);
+	trans1->attached(true);
 	trans1->deref();
     }
     else
