@@ -256,7 +256,7 @@ void MessageHandler::destruct()
     String::destruct();
 }
 
-void MessageHandler::safeNow()
+void MessageHandler::safeNowInternal()
 {
     Lock lock(m_dispatcher);
     // when the unsafe counter reaches zero we're again safe to destroy
@@ -266,7 +266,7 @@ void MessageHandler::safeNow()
 bool MessageHandler::receivedInternal(Message& msg)
 {
     bool ok = received(msg);
-    safeNow();
+    safeNowInternal();
     return ok;
 }
 
@@ -290,7 +290,7 @@ bool MessageRelay::receivedInternal(Message& msg)
 {
     MessageReceiver* receiver = m_receiver;
     int id = m_id;
-    safeNow();
+    safeNowInternal(); // At this point the relay itself may be uninstalled
     return receiver && receiver->received(msg,id);
 }
 
@@ -300,6 +300,8 @@ MessageDispatcher::MessageDispatcher(const char* trackParam)
       m_hookMutex(false,"PostHooks"),
       m_msgAppend(&m_messages), m_hookAppend(&m_hooks),
       m_trackParam(trackParam), m_changes(0), m_warnTime(0),
+      m_enqueueCount(0), m_dequeueCount(0), m_dispatchCount(0),
+      m_queuedMax(0), m_msgAvgAge(0),
       m_hookCount(0), m_hookHole(false)
 {
     XDebug(DebugInfo,"MessageDispatcher::MessageDispatcher('%s') [%p]",trackParam,this);
@@ -389,6 +391,7 @@ bool MessageDispatcher::dispatch(Message& msg)
     NamedCounter* saved = Thread::getCurrentObjCounter(counting);
     ObjList *l = &m_handlers;
     Lock mylock(this);
+    m_dispatchCount++;
     for (; l; l=l->next()) {
 	MessageHandler *h = static_cast<MessageHandler*>(l->get());
 	if (h && (h->null() || *h == msg)) {
@@ -519,6 +522,9 @@ bool MessageDispatcher::enqueue(Message* msg)
     if (!msg || m_messages.find(msg))
 	return false;
     m_msgAppend = m_msgAppend->append(msg);
+    u_int64_t count = (++m_enqueueCount) - m_dequeueCount;
+    if (m_queuedMax < count)
+	m_queuedMax = count;
     return true;
 }
 
@@ -528,6 +534,12 @@ bool MessageDispatcher::dequeueOne()
     if (m_messages.next() == m_msgAppend)
 	m_msgAppend = &m_messages;
     Message* msg = static_cast<Message *>(m_messages.remove(false));
+    if (msg) {
+	m_dequeueCount++;
+	uint64_t age = Time::now() - msg->msgTime();
+	if (age < 60000000)
+	    m_msgAvgAge = (3 * m_msgAvgAge + age) >> 2;
+    }
     unlock();
     if (!msg)
 	return false;
@@ -545,7 +557,7 @@ void MessageDispatcher::dequeue()
 unsigned int MessageDispatcher::messageCount()
 {
     Lock lock(this);
-    return m_messages.count();
+    return (unsigned int)(m_enqueueCount - m_dequeueCount);
 }
 
 unsigned int MessageDispatcher::handlerCount()
@@ -560,6 +572,16 @@ unsigned int MessageDispatcher::postHookCount()
     return m_hooks.count();
 }
 
+void MessageDispatcher::getStats(u_int64_t& enqueued, u_int64_t& dequeued, u_int64_t& dispatched, u_int64_t& queueMax)
+{
+    lock();
+    enqueued = m_enqueueCount;
+    dequeued = m_dequeueCount;
+    dispatched = m_dispatchCount;
+    queueMax = m_queuedMax;
+    unlock();
+}
+
 void MessageDispatcher::setHook(MessagePostHook* hook, bool remove)
 {
     m_hookMutex.lock();
