@@ -1454,7 +1454,7 @@ bool File::md5(const char* name, String& buffer, int* error)
 }
 
 // Create a folder (directory)
-bool File::mkDir(const char* path, int* error)
+bool File::mkDir(const char* path, int* error, int mode)
 {
     if (!fileNameOk(path,error))
 	return false;
@@ -1462,7 +1462,7 @@ bool File::mkDir(const char* path, int* error)
     if (::CreateDirectoryA(path,NULL))
 	return true;
 #else
-    if (0 == ::mkdir(path,(mode_t)-1))
+    if (0 == ::mkdir(path,(mode_t)mode))
 	return true;
 #endif
     return getLastError(error);
@@ -2028,7 +2028,7 @@ bool Socket::select(bool* readok, bool* writeok, bool* except, struct timeval* t
     if (tmp >= (SOCKET)FD_SETSIZE) {
 	if (localFail) {
 	    localFail = false;
-	    Alarm("engine","socket",DebugGoOn,"Socket::select: handle %d larger than compiled in maximum %d",
+	    Alarm("engine","socket",DebugCrit,"Socket::select: handle %d larger than compiled in maximum %d",
 		tmp,FD_SETSIZE);
 	}
 	return false;
@@ -2117,6 +2117,27 @@ bool Socket::setTOS(int tos)
 #endif
 }
 
+int Socket::getTOS()
+{
+    int tos = Normal;
+    socklen_t len = sizeof(tos);
+#if defined(AF_INET6) && defined(IPV6_TCLASS)
+    SocketAddr addr;
+    if (getSockName(addr) && addr.family() == AF_INET6) {
+	if (getOption(IPPROTO_IPV6,IPV6_TCLASS,&tos,&len))
+	    return tos;
+	tos = Normal;
+	len = sizeof(tos);
+    }
+#endif
+#ifdef IP_TOS
+    getOption(IPPROTO_IP,IP_TOS,&tos,&len);
+#else
+    m_error = ENOTIMPL;
+#endif
+    return tos;
+}
+
 bool Socket::setBlocking(bool block)
 {
     unsigned long flags = 1;
