@@ -104,6 +104,32 @@ const char* lookup(int value, const TokenDict* tokens, const char* defvalue)
     return defvalue;
 }
 
+int64_t lookup(const char* str, const TokenDict64* tokens, int64_t defvalue, int base)
+{
+    if (!str)
+	return defvalue;
+    if (tokens) {
+	for (; tokens->token; tokens++)
+	    if (!::strcmp(str,tokens->token))
+		return tokens->value;
+    }
+    char* eptr = 0;
+    int64_t val = ::strtoll(str,&eptr,base);
+    if (!eptr || *eptr)
+	return defvalue;
+    return val;
+}
+
+const char* lookup(int64_t value, const TokenDict64* tokens, const char* defvalue)
+{
+    if (tokens) {
+	for (; tokens->token; tokens++)
+	    if (value == tokens->value)
+		return tokens->token;
+    }
+    return defvalue;
+}
+
 #define MAX_MATCH 9
 
 class StringMatchPrivate
@@ -1517,6 +1543,25 @@ ObjList* String::split(char separator, bool emptyOK) const
     return list;
 }
 
+ObjList* String::split(const Regexp& reg, bool emptyOK) const
+{
+    String s = *this;
+    ObjList* list = new ObjList;
+    ObjList* dest = list;
+    while (true) {
+	if (!(s && s.matches(reg))) {
+	    if (s || emptyOK)
+		dest = dest->append(new String(s));
+	    break;
+	}
+	int pos = s.matchOffset(0);
+	if (emptyOK || pos > 0)
+	    dest = dest->append(new String(s.c_str(),pos));
+	s = s.substr(pos + s.matchLength(0));
+    }
+    return list;
+}
+
 String String::msgEscape(const char* str, char extraEsc)
 {
     String s;
@@ -2057,4 +2102,56 @@ const String& GenObject::toString() const
     return String::empty();
 }
 
+unsigned int String::encodeFlags(const TokenDict* tokens) const
+{
+    unsigned int flags = 0;
+    ObjList* list = split(',',false);
+    for (ObjList* o = list->skipNull(); o; o = o->skipNext()) {
+	flags |= (unsigned int)(lookup(*static_cast<String*>(o->get()),tokens));
+    }
+    TelEngine::destruct(list);
+    return flags;
+}
+
+uint64_t String::encodeFlags(const TokenDict64* tokens) const
+{
+    uint64_t flags = 0;
+    ObjList* list = split(',',false);
+    for (ObjList* o = list->skipNull(); o; o = o->skipNext()) {
+	flags |= (uint64_t)(lookup(*static_cast<String*>(o->get()),tokens));
+    }
+    TelEngine::destruct(list);
+    return flags;
+}
+
+const String& String::decodeFlags(unsigned int flags, const TokenDict* tokens, bool unknownflag)
+{
+    if (tokens) {
+	for(; tokens->token && flags; tokens++) {
+	    if ((tokens->value & flags) == (unsigned int)tokens->value) {
+		append(tokens->token,",");
+		flags &= ~tokens->value;
+	    }
+	}
+    }
+    if (flags && unknownflag)
+	append(String(flags),",");
+    return *this;
+}
+
+const String& String::decodeFlags(uint64_t flags, const TokenDict64* tokens, bool unknownflag)
+{
+    if (tokens) {
+	for (; tokens->token && flags; tokens++) {
+	    if ((tokens->value & flags) == (uint64_t)tokens->value ) {
+		append(tokens->token,",");
+		flags &= ~tokens->value;
+	    }
+	}
+    }
+    if (flags && unknownflag)
+	append(String(flags),",");
+    return *this;
+}
+
 /* vi: set ts=8 sw=4 sts=4 noet: */
