@@ -52,7 +52,7 @@ bool ScriptParser::parseFile(const char* name, bool fragment)
     if (!f.openPath(name))
 	return false;
     int64_t len = f.length();
-    if (len <= 0 || len > 262143)
+    if (len <= 0 || len > (int64_t)m_maxFileLen)
 	return false;
     DataBlock data(0,(unsigned int)len+1);
     char* text = (char*)data.data();
@@ -103,6 +103,8 @@ void* ScriptContext::getObject(const String& name) const
 	return const_cast<ScriptContext*>(this);
     if (name == YATOM("ExpExtender"))
 	return const_cast<ExpExtender*>(static_cast<const ExpExtender*>(this));
+    if (name == YATOM("NamedList"))
+	return const_cast<NamedList*>(&m_params);
     return RefObject::getObject(name);
 }
 
@@ -180,9 +182,22 @@ bool ScriptContext::copyFields(ObjList& stack, const ScriptContext& original, Ge
     return ok;
 }
 
+// Add string parameters from list
+void ScriptContext::addFields(const NamedList& list, const char* skipPrefix)
+{
+    if (TelEngine::null(skipPrefix))
+	skipPrefix = 0;
+    for (const ObjList* o = list.paramList()->skipNull(); o; o = o->skipNext()) {
+	const NamedString* p = static_cast<const NamedString*>(o->get());
+	if (!(skipPrefix && p->name().startsWith(skipPrefix)))
+	    m_params.addParam(new ExpOperation(p->c_str(),p->name()));
+    }
+}
+
 void ScriptContext::fillFieldNames(ObjList& names)
 {
-    fillFieldNames(names,params());
+    bool checkDupl = !(YOBJECT(JsObject,this));
+    fillFieldNames(names,params(),checkDupl);
     const NamedList* native = nativeParams();
     if (native)
 	fillFieldNames(names,*native);
@@ -193,23 +208,34 @@ void ScriptContext::fillFieldNames(ObjList& names)
 #endif
 }
 
-void ScriptContext::fillFieldNames(ObjList& names, const NamedList& list, const char* skip)
+void ScriptContext::fillFieldNames(ObjList& names, const NamedList& list, bool checkDupl, const char* skip)
 {
-    unsigned int n = list.length();
-    for (unsigned int i = 0; i < n; i++) {
-	const NamedString* s = list.getParam(i);
-	if (!s)
-	    continue;
+    ObjList* tail = &names;
+    for (const ObjList* l = list.paramList()->skipNull(); l; l = l->skipNext()) {
+	const NamedString* s = static_cast<const NamedString*>(l->get());
 	if (s->name().null())
 	    continue;
 	if (skip && s->name().startsWith(skip))
 	    continue;
-	if (names.find(s->name()))
+	if (checkDupl && names.find(s->name()))
 	    continue;
-	names.append(new String(s->name()));
+	tail = tail->append(new String(s->name()));
     }
 }
 
+void ScriptContext::fillFieldNames(ObjList& names, const HashList& list)
+{
+    ObjList* tail = &names;
+    for (unsigned int i = 0; i < list.length(); i++) {
+	ObjList* o = list.getList(i);
+	for (o = o ? o->skipNull() : 0;o;o = o->skipNext()) {
+	    GenObject* obj = o->get();
+	    if (obj->toString().null())
+		continue;
+	    tail = tail->append(new String(obj->toString()));
+	}
+    }
+}
 
 #define MAKE_NAME(x) { #x, ScriptRun::x }
 static const TokenDict s_states[] = {
@@ -242,6 +268,8 @@ ScriptRun::ScriptRun(ScriptCode* code, ScriptContext* context)
 ScriptRun::~ScriptRun()
 {
     XDebug(DebugAll,"ScriptRun::~ScriptRun [%p]",this);
+    m_stack.clear();
+    m_async.clear();
     lock();
     m_state = Invalid;
     TelEngine::destruct(m_code);
