@@ -34,6 +34,7 @@ const TokenDict SDPParser::s_payloads[] = {
     { "lpc10",         7 },
     { "slin",         11 },
     { "g726",          2 },
+    { "g722/16000",    9 },
     { "g722",          9 },
     { "g723",          4 },
     { "g728",         15 },
@@ -59,6 +60,7 @@ const TokenDict SDPParser::s_payloads[] = {
     { "h263-2000",   112 },
     { "h264",        114 },
     { "vp8",         113 },
+    { "vp9",         115 },
     { "mpv",          32 },
     { "mp2t",         33 },
     { "mp4v",        110 },
@@ -95,6 +97,7 @@ const TokenDict SDPParser::s_rtpmap[] = {
     { "H263-2000/90000",  112 },
     { "H264/90000",       114 },
     { "VP8/90000",        113 },
+    { "VP9/90000",        115 },
     { "MPV/90000",         32 },
     { "MP2T/90000",        33 },
     { "MP4V-ES/90000",    110 },
@@ -211,12 +214,6 @@ ObjList* SDPParser::parse(const MimeSdpBody& sdp, String& addr, ObjList* oldMedi
 	    bool amrOctet = m_codecs.getBoolValue("amr_octet",false);
 	    int defmap = -1;
 	    String payload(lookup(var,s_payloads));
-	    if (amrOctet) {
-		if (payload == YSTRING("amr"))
-		    payload = "amr-o";
-		else if (payload == YSTRING("amr/16000"))
-		    payload = "amr-o/16000";
-	    }
 
 	    const ObjList* l = sdp.lines().find(c);
 	    while (l && (l = l->skipNext())) {
@@ -262,6 +259,12 @@ ObjList* SDPParser::parse(const MimeSdpBody& sdp, String& addr, ObjList* oldMedi
 			    }
 			}
 			payload = pload;
+			if (amrOctet) {
+			    if (payload == YSTRING("amr"))
+				payload = "amr-o";
+			    else if (payload == YSTRING("amr/16000"))
+				payload = "amr-o/16000";
+			}
 		    }
 		}
 		else if (line.startSkip("fmtp:",false)) {
