@@ -207,7 +207,7 @@ SIPTransaction* SIPEngine::addMessage(SIPParty* ep, const char* buf, int len)
     return 0;
 }
 
-SIPTransaction* SIPEngine::addMessage(SIPMessage* message)
+SIPTransaction* SIPEngine::addMessage(SIPMessage* message, bool* autoChangeParty)
 {
     DDebug(this,DebugInfo,"addMessage(%p) [%p]",message,this);
     if (!message)
@@ -257,7 +257,7 @@ SIPTransaction* SIPEngine::addMessage(SIPMessage* message)
 	return 0;
     }
     message->complete(this);
-    return new SIPTransaction(message,this,message->isOutgoing());
+    return new SIPTransaction(message,this,message->isOutgoing(),autoChangeParty);
 }
 
 SIPTransaction* SIPEngine::forkInvite(SIPMessage* answer, SIPTransaction* trans)
@@ -314,6 +314,7 @@ void SIPEngine::processEvent(SIPEvent *event)
 {
     if (!event)
 	return;
+#ifdef DEBUG
     const char* type = "unknown";
     if (event->isOutgoing())
 	type = "outgoing";
@@ -321,6 +322,7 @@ void SIPEngine::processEvent(SIPEvent *event)
 	type = "incoming";
     DDebug(this,DebugAll,"Processing %s event %p message %p [%p]",
 	type,event,event->getMessage(),this);
+#endif
     if (event->getMessage()) {
 	if (event->isOutgoing()) {
 	    switch (event->getState()) {
