@@ -267,7 +267,7 @@ void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domai
     if (!isAnswer()) {
 	hl = const_cast<MimeHeaderLine*>(getHeader("From"));
 	if (!hl) {
-	    String tmp = "<sip:";
+	    String tmp = uri.startsWith("sips:") ? "<sips:" : "<sip:";
 	    if (user)
 		tmp << String::uriEscape(user,'@',"+?&") << "@";
 	    tmp << domain << ">";
@@ -300,7 +300,7 @@ void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domai
 	    if (m_cseq <= 0) {
 		int sep = hl->find(' ');
 		if (sep > 0)
-		    m_cseq = hl->substr(sep).toInteger(-1,10);
+		    m_cseq = hl->substr(0,sep).toInteger(-1,10);
 	    }
 	}
 	else {
@@ -334,8 +334,8 @@ void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domai
 	addHeader("Max-Forwards",tmp);
     }
 
-    if ((method == YSTRING("INVITE")) && !getHeader("Contact")) {
-	// automatically add a contact field to (re)INVITE and its answers
+    if ((method == YSTRING("INVITE") || method == YSTRING("UPDATE")) && !getHeader("Contact")) {
+	// automatically add a contact field to (re)INVITE, UPDATE and their answers
 	String tmp(user);
 	if (!tmp) {
 	    tmp = uri;
@@ -345,7 +345,7 @@ void SIPMessage::complete(SIPEngine* engine, const char* user, const char* domai
 	}
 	if (tmp)
 	    tmp = tmp.uriEscape('@',"+?&") + "@";
-	tmp = "<sip:" + tmp;
+	tmp = (uri.startsWith("sips:") ? "<sips:" : "<sip:") + tmp;
 	SocketAddr::appendTo(tmp,partyLAddr,partyLPort) << ">";
 	addHeader("Contact",tmp);
     }
