@@ -63,6 +63,7 @@ static Mutex s_mutex(true,"ExtModule");
 static Mutex s_uses(false,"ExtModUse");
 static int s_waitFlush = WAIT_FLUSH;
 static int s_timeout = MSG_TIMEOUT;
+static bool s_settime = false;
 static bool s_timebomb = false;
 static bool s_pluginSafe = true;
 static const char* s_trackName = 0;
@@ -272,6 +273,7 @@ private:
     bool m_selfWatch;
     bool m_reenter;
     bool m_setdata;
+    bool m_settime;
     bool m_writing;
     int m_timeout;
     bool m_timebomb;
@@ -789,7 +791,7 @@ ExtModReceiver::ExtModReceiver(const char* script, const char* args, File* ain,
       m_role(RoleUnknown), m_dead(false), m_quit(false), m_use(1), m_pid(-1),
       m_in(0), m_out(0), m_ain(ain), m_aout(aout),
       m_chan(chan), m_watcher(0),
-      m_selfWatch(false), m_reenter(false), m_setdata(true), m_writing(false),
+      m_selfWatch(false), m_reenter(false), m_setdata(true), m_settime(s_settime), m_writing(false),
       m_timeout(s_timeout), m_timebomb(s_timebomb), m_restart(false), m_scripted(false),
       m_buffer(0,DEF_INCOMING_LINE), m_script(script), m_args(args), m_trackName(s_trackName)
 {
@@ -807,7 +809,7 @@ ExtModReceiver::ExtModReceiver(const char* name, Stream* io, ExtModChan* chan, i
       m_role(role), m_dead(false), m_quit(false), m_use(1), m_pid(-1),
       m_in(io), m_out(io), m_ain(0), m_aout(0),
       m_chan(chan), m_watcher(0),
-      m_selfWatch(false), m_reenter(false), m_setdata(true), m_writing(false),
+      m_selfWatch(false), m_reenter(false), m_setdata(true), m_settime(s_settime), m_writing(false),
       m_timeout(s_timeout), m_timebomb(s_timebomb), m_restart(false), m_scripted(false),
       m_buffer(0,DEF_INCOMING_LINE), m_script(name), m_args(conn), m_trackName(s_trackName)
 {
@@ -988,7 +990,12 @@ bool ExtModReceiver::received(Message &msg, int id)
 {
     if (m_dead || m_quit)
 	return false;
-    lock();
+    if (!lock(m_timeout > 0 ? ((long)m_timeout * 1000) : -1)) {
+	Alarm("extmodule","performance",DebugWarn,
+	    "Failed to lock to queue message (%p) '%s' for %d msec [%p]",
+	    &msg,msg.c_str(),m_timeout,this);
+	return false;
+    }
     // check if we are no longer running
     bool ok = (m_pid > 0) && !m_dead && m_in && m_in->valid() && m_out && m_out->valid();
     if (ok && !m_reenter) {
@@ -1168,10 +1175,16 @@ void ExtModReceiver::run()
 	use();
 	lock();
 	char* buffer = static_cast<char*>(m_buffer.data());
-	int readsize = m_in ? m_in->readData(buffer+posinbuf,m_buffer.length()-posinbuf) : 0;
+	int bufspace = m_buffer.length() - posinbuf - 1;
+	int readsize = (m_in && bufspace) ? m_in->readData(buffer+posinbuf,bufspace) : 0;
 	unlock();
 	if (unuse())
 	    return;
+	if (!bufspace) {
+	    Debug("ExtModule",DebugWarn,"Overflow reading in buffer of length %u, closing [%p]",
+		m_buffer.length(),this);
+	    return;
+	}
 	if (!readsize) {
 	    if (m_in)
 		Debug("ExtModule",DebugInfo,"Read EOF on %p [%p]",m_in,this);
@@ -1202,7 +1215,7 @@ void ExtModReceiver::run()
 		m_buffer.length(),this);
 	    return;
 	}
-	buffer[totalsize]=0;
+	buffer[totalsize] = 0;
 	for (;;) {
 	    char *eoline = ::strchr(buffer,'\n');
 	    if (!eoline && ((int)::strlen(buffer) < totalsize))
@@ -1542,6 +1555,11 @@ bool ExtModReceiver::processLine(const char* line)
 		val = m_setdata;
 		ok = true;
 	    }
+	    else if (id == "settime") {
+		m_settime = val.toBoolean(m_settime);
+		val = m_setdata;
+		ok = true;
+	    }
 	    else if (id == "selfwatch") {
 		m_selfWatch = val.toBoolean(m_selfWatch);
 		val = m_selfWatch;
@@ -1637,6 +1655,8 @@ bool ExtModReceiver::processLine(const char* line)
 		    }
 		}
 	    }
+	    if (m_settime || !m->msgTime())
+		m->msgTime() = Time::now();
 	    m->startup(this);
 	    unlock();
 	    return false;
@@ -2020,6 +2040,7 @@ void ExtModulePlugin::initialize()
     s_cfg.load();
     s_timeout = s_cfg.getIntValue("general","timeout",MSG_TIMEOUT);
     s_timebomb = s_cfg.getBoolValue("general","timebomb",false);
+    s_settime = s_cfg.getBoolValue("general","settime",false);
     s_trackName = s_cfg.getBoolValue("general","trackparam",false) ?
 	name().c_str() : (const char*)0;
     int wf = s_cfg.getIntValue("general","waitflush",WAIT_FLUSH);
