@@ -30,12 +30,23 @@
 
 #ifndef OPENSSL_NO_AES
 #include <openssl/aes.h>
+#ifdef NO_AESCTR
+#include <openssl/modes.h>
+#define AES_ctr128_encrypt(in,out,len,key,ivec,ecount,num) \
+     CRYPTO_ctr128_encrypt(in,out,len,key,ivec,ecount,num,(block128_f)AES_encrypt)
+#endif
 #endif
 
 #ifndef OPENSSL_NO_DES
 #include <openssl/des.h>
 #endif
 
+#ifdef USE_TLS_METHOD
+#define CTX_METHOD ::TLS_method()
+#else
+#define CTX_METHOD ::SSLv23_method()
+#endif
+
 using namespace TelEngine;
 namespace { // anonymous
 
@@ -286,7 +297,7 @@ SslContext::SslContext(const char* name)
     : String(name),
     m_context(0)
 {
-    m_context = ::SSL_CTX_new(::SSLv23_method());
+    m_context = ::SSL_CTX_new(CTX_METHOD);
     SSL_CTX_set_info_callback(m_context,infoCallback);
 #ifdef DEBUG
     SSL_CTX_set_msg_callback(m_context,msgCallback);
@@ -418,7 +429,7 @@ SslSocket::SslSocket(SOCKET handle, bool server, int verify, SslContext* context
 	::SSL_set_fd(m_ssl,handle);
 	BIO* bio = ::SSL_get_rbio(m_ssl);
 	if (!(bio && BIO_set_close(bio,BIO_NOCLOSE)))
-	    Debug(&__plugin,DebugGoOn,"SslSocket::SslSocket(%d) no BIO or cannot set NOCLOSE [%p]",
+	    Debug(&__plugin,DebugCrit,"SslSocket::SslSocket(%d) no BIO or cannot set NOCLOSE [%p]",
 		handle,this);
 	if (server)
 	    ::SSL_set_accept_state(m_ssl);
@@ -555,12 +566,12 @@ bool SslHandler::received(Message& msg)
     addRand(msg.msgTime());
     Socket** ppSock = static_cast<Socket**>(msg.userObject(YATOM("Socket*")));
     if (!ppSock) {
-	Debug(&__plugin,DebugGoOn,"SslHandler: No pointer to Socket");
+	Debug(&__plugin,DebugCrit,"SslHandler: No pointer to Socket");
 	return false;
     }
     Socket* pSock = *ppSock;
     if (!pSock) {
-	Debug(&__plugin,DebugGoOn,"SslHandler: NULL Socket pointer");
+	Debug(&__plugin,DebugCrit,"SslHandler: NULL Socket pointer");
 	return false;
     }
     if (!pSock->valid()) {
@@ -901,11 +912,13 @@ void OpenSSL::initialize()
     Configuration cfg(Engine::configFile("openssl"));
     if (!m_handler) {
 	setup();
+#ifndef NO_LOAD_ERR
 	::SSL_load_error_strings();
 	::SSL_library_init();
+#endif
 	addRand(Time::now());
 	s_index = ::SSL_get_ex_new_index(0,const_cast<char*>("TelEngine::SslSocket"),0,0,0);
-	s_context = ::SSL_CTX_new(::SSLv23_method());
+	s_context = ::SSL_CTX_new(CTX_METHOD);
 	SSL_CTX_set_info_callback(s_context,infoCallback); // macro - no ::
 	m_handler = new SslHandler;
 	Engine::install(m_handler);
