@@ -381,6 +381,8 @@ static void evalFunc(String& str, Message& msg)
 		str = msg.msgTime().sec();
 	    else if (par == YSTRING("broadcast"))
 		str = msg.broadcast();
+	    else if (par == YSTRING("retval"))
+		str = msg.retValue();
 	    else if (par == YSTRING("count"))
 		str = msg.count();
 	    else if (par == YSTRING("parameters")) {
@@ -428,6 +430,8 @@ static void evalFunc(String& str, Message& msg)
 	    str = Engine::nodeName();
 	else if (str == YSTRING("threadname"))
 	    str = Thread::currentName();
+	else if (str == YSTRING("accepting"))
+	    str = lookup(Engine::accept(),Engine::getCallAcceptStates());
 	else if ((sep >= 0) && (str == YSTRING("transcode"))) {
 	    str = par.substr(0,sep);
 	    par = par.substr(sep+1).trimBlanks();
@@ -441,6 +445,11 @@ static void evalFunc(String& str, Message& msg)
 	}
 	else if (str == YSTRING("dispatching"))
 	    str = s_dispatching;
+	else if (str == YSTRING("timestamp")) {
+	    char buf[24];
+	    Debugger::formatTime(buf);
+	    str = buf;
+	}
 	else if (bare && str.trimBlanks())
 	    str = s_vars.getValue(str);
 	else {
@@ -624,7 +633,7 @@ static bool oneContext(Message &msg, String &str, const String &context, String
 	    BlockState blockThis = (blockDepth > 0) ? blockStack[blockDepth-1] : BlockRun;
 	    BlockState blockLast = BlockSkip;
 	    Regexp reg(n->name(),s_extended,s_insensitive);
-	    if (reg.startSkip("}")) {
+	    if (reg.startSkip("}",false)) {
 		if (!blockDepth) {
 		    Debug("RegexRoute",DebugWarn,"Got '}' outside block in line #%u in context '%s'",
 			i+1,context.c_str());
@@ -636,7 +645,7 @@ static bool oneContext(Message &msg, String &str, const String &context, String
 		blockLast = blockThis;
 		blockThis = (blockDepth > 0) ? blockStack[blockDepth-1] : BlockRun;
 	    }
-	    static Regexp s_blockStart("\\(=[[:space:]]*\\)\\?{$");
+	    static Regexp s_blockStart("^\\(.*=[[:space:]]*\\)\\?{$");
 	    if (s_blockStart.matches(*n)) {
 		// start of a new block
 		if (blockDepth >= BLOCK_STACK) {
@@ -647,7 +656,7 @@ static bool oneContext(Message &msg, String &str, const String &context, String
 		// assume block is done
 		BlockState blockEnter = BlockDone;
 		if (BlockRun == blockThis) {
-		    // if we returned from a false inner block to a true outer block
+		    // if we just returned from a false inner block to a true outer block
 		    if (BlockSkip == blockLast)
 			blockEnter = BlockSkip;
 		    else
@@ -655,6 +664,8 @@ static bool oneContext(Message &msg, String &str, const String &context, String
 		}
 		blockStack[blockDepth++] = blockEnter;
 	    }
+	    else if (BlockSkip != blockLast)
+		blockThis = BlockDone;
 	    XDebug("RegexRoute",DebugAll,"%s:%d(%u:%s) %s=%s",context.c_str(),i+1,
 		blockDepth,String::boolText(BlockRun == blockThis),
 		n->name().c_str(),n->c_str());
@@ -708,12 +719,25 @@ static bool oneContext(Message &msg, String &str, const String &context, String
 	    if (!ok)
 		continue;
 
-	    if (val.startSkip("echo") || val.startSkip("output")) {
+	    int level = 0;
+	    if (val.startSkip("echo") || val.startSkip("output")
+		    || (val.startSkip("debug") && ((level = DebugAll)))) {
+		if (level) {
+		    val >> level;
+		    val.trimBlanks();
+		    if (level < DebugTest)
+			level = DebugTest;
+		    else if (level > DebugAll)
+			level = DebugAll;
+		}
 		// special case: display the line but don't set params
 		val = match.replaceMatches(val);
 		msg.replaceParams(val);
 		replaceFuncs(val,msg);
-		Output("%s",val.safe());
+		if (level)
+		    Debug(level,"%s",val.safe());
+		else
+		    Output("%s",val.safe());
 		continue;
 	    }
 	    else if (val == "{") {
@@ -756,7 +780,7 @@ static bool oneContext(Message &msg, String &str, const String &context, String
 	    setMessage(match,msg,val);
 	    warn = true;
 	    val.trimBlanks();
-	    if (val.null()) {
+	    if (val.null() || val.startSkip("noop")) {
 		// special case: do nothing on empty target
 		continue;
 	    }
@@ -795,6 +819,11 @@ static bool oneContext(Message &msg, String &str, const String &context, String
 		    msg = val;
 		}
 	    }
+	    else if (val.startSkip("retval")) {
+		NDebug("RegexRoute",DebugAll,"Setting retValue length %u by rule #%u '%s' in context '%s'",
+			val.length(),i+1,n->name().c_str(),context.c_str());
+		ret = val;
+	    }
 	    else {
 		DDebug("RegexRoute",DebugAll,"Returning '%s' for '%s' in context '%s' by rule #%u '%s'",
 		    val.c_str(),str.c_str(),context.c_str(),i+1,n->name().c_str());
