@@ -92,7 +92,6 @@ public:
     virtual void run();
 private:
     RudpSocket* m_rudp;                                    // The rudp socket that holds this thread
-    bool m_stop;
 };
 
 class RudpSocket : public GenObject, public Mutex
@@ -394,6 +393,7 @@ public:
     void processSltMessage(u_int16_t msgType, DataBlock& data);
     void buildHeader(DataBlock& data, bool management = false);
     void sendConnect(unsigned int status);
+    void sendAutoConnect();
     void sendControllerR(unsigned int linkState);
     void sendManagement(unsigned int message);
     void sendDisconnect();
@@ -469,7 +469,7 @@ YSIGFACTORY2(SLT);
 
 RudpThread::RudpThread(RudpSocket* rudp, Priority prio)
     : Thread("RUDP Runner",prio),
-    m_rudp(rudp), m_stop(false)
+    m_rudp(rudp)
 {
 }
 
@@ -912,7 +912,8 @@ bool RudpSocket::readData()
     if (m_state == RudpDown && !haveSyn((u_int8_t)packet.at(0)))
 	return false;
     if (m_haveChecksum && !checkChecksum(packet)) {
-	DDebug(m_sm,DebugMild,"Wrong checksum received");
+	m_wrongChecksum++;
+	DDebug(m_sm,DebugMild,"Wrong checksums received: %u",m_wrongChecksum);
 	return false;
     }
     recvMsg(packet);
@@ -1888,10 +1889,7 @@ void SLT::processSltMessage(u_int16_t msgType, DataBlock& data)
 	case Disconnect_C:
 	case Disconnect_I:
 	    setRemoteStatus(OutOfService);
-	    if (!m_autostart)
-		break;
-	    if (m_reqStatus == EmergencyAlignment || m_reqStatus == NormalAlignment)
-		sendConnect(m_reqStatus == NormalAlignment ? Normal : Emergency);
+	    sendAutoConnect();
 	    break;
 	case Link_State_Controller_C:
 	    setRemoteStatus(m_reqStatus);
@@ -1947,6 +1945,17 @@ void SLT::sendConnect(unsigned int status)
     m_session->sendData(data,true);
 }
 
+void SLT::sendAutoConnect()
+{
+    if (!m_autostart)
+	return;
+    if (m_reqStatus != EmergencyAlignment && m_reqStatus != NormalAlignment)
+	return;
+    if (m_autoEmergency)
+	setReqStatus(getEmergency() ? EmergencyAlignment : NormalAlignment);
+    sendConnect(m_reqStatus == EmergencyAlignment ? Emergency : Normal);
+}
+
 void SLT::sendControllerR(unsigned int linkState)
 {
     DataBlock data;
@@ -2073,10 +2082,7 @@ void SLT::configure(bool start)
     setStatus(Configured);
     SS7Layer2::notify();
     DDebug(this,DebugInfo,"requested status = %s",statusName(m_reqStatus,false));
-    if (!m_autostart)
-	return;
-    if (m_reqStatus == NormalAlignment || m_reqStatus == EmergencyAlignment)
-	sendConnect(m_reqStatus == NormalAlignment ? Normal : Emergency);
+    sendAutoConnect();
 }
 
 SignallingComponent* SLT::create(const String& type, NamedList& name)
