@@ -38,7 +38,6 @@ namespace { // anonymous
 class Transport;
 class TransportWorker;
 class TransportThread;
-class SockRef;
 class MessageReader;
 class TReader;
 class StreamReader;
@@ -78,22 +77,6 @@ private:
     Mutex m_threadMutex;
 };
 
-class SockRef : public RefObject
-{
-public:
-    inline SockRef(Socket** sock)
-	: m_sock(sock)
-	{ }
-    void* getObject(const String& name) const
-    {
-	if (name == "Socket*")
-	    return m_sock;
-	return RefObject::getObject(name);
-    }
-private:
-    Socket** m_sock;
-};
-
 class TransportThread : public Thread
 {
     friend class TransportWorker;
@@ -396,7 +379,7 @@ bool ListenerThread::init(const NamedList& param)
 	{
 	    Socket* soc = 0;
 	    Message m("socket.sctp");
-	    SockRef* s = new SockRef(&soc);
+	    SocketRef* s = new SocketRef(soc);
 	    m.userData(s);
 	    TelEngine::destruct(s);
 	    if (!(Engine::dispatch(m) && soc)) {
@@ -501,7 +484,7 @@ bool ListenerThread::addAddress(const NamedList &param)
     }
     SctpSocket* s = static_cast<SctpSocket*>(m_socket);
     if (!s) {
-	Debug("ListenerThread",DebugGoOn,"Failed to cast socket");
+	Debug("ListenerThread",DebugCrit,"Failed to cast socket");
 	return false;
     }
     if (!s->bindx(o)) {
@@ -780,7 +763,7 @@ bool Transport::bindSocket()
 	{
 	    Socket* soc = 0;
 	    Message m("socket.sctp");
-	    SockRef* s = new SockRef(&soc);
+	    SocketRef* s = new SocketRef(soc);
 	    m.userData(s);
 	    TelEngine::destruct(s);
 	    if (!(Engine::dispatch(m) && soc)) {
@@ -884,7 +867,7 @@ bool Transport::addAddress(const NamedList &param, Socket* socket)
     }
     SctpSocket* s = static_cast<SctpSocket*>(socket);
     if (!s) {
-	Debug(this,DebugGoOn,"Failed to cast socket");
+	Debug(this,DebugCrit,"Failed to cast socket");
 	return false;
     }
     if (!s->bindx(o)) {
@@ -905,7 +888,7 @@ bool Transport::connectSocket()
 	case Sctp :
 	{
 	    Message m("socket.sctp");
-	    SockRef* s = new SockRef(&sock);
+	    SocketRef* s = new SocketRef(sock);
 	    m.userData(s);
 	    TelEngine::destruct(s);
 	    if (!(Engine::dispatch(m) && sock)) {
@@ -992,7 +975,7 @@ bool Transport::connectSocket()
 	}
 	SctpSocket* s = static_cast<SctpSocket*>(sock);
 	if (!s) {
-	    Debug(this,DebugGoOn,"Failed to cast socket");
+	    Debug(this,DebugCrit,"Failed to cast socket");
 	    return false;
 	}
 	if (!s->connectx(o)) {
@@ -1088,7 +1071,7 @@ bool Transport::addSocket(Socket* socket,SocketAddr& socketAddress)
 	    Message m("socket.sctp");
 	    m.addParam("handle",String(socket->detach()));
 	    delete socket;
-	    SockRef* s = new SockRef(&sock);
+	    SocketRef* s = new SocketRef(sock);
 	    m.userData(s);
 	    TelEngine::destruct(s);
 	    if (!(Engine::dispatch(m) && sock)) {
@@ -1228,7 +1211,7 @@ bool StreamReader::sendBuffer(int streamId)
     if (m_transport->transType() == Transport::Sctp) {
 	SctpSocket* s = static_cast<SctpSocket*>(m_socket);
 	if (!s) {
-	    Debug(m_transport,DebugGoOn,"Sctp conversion failed");
+	    Debug(m_transport,DebugCrit,"Sctp conversion failed");
 	    return false;
 	}
 	if (m_transport->status() == Transport::Up)
@@ -1310,7 +1293,7 @@ bool StreamReader::readData()
 	if (m_transport->transType() == Transport::Sctp) {
 	    SctpSocket* s = static_cast<SctpSocket*>(m_socket);
 	    if (!s) {
-		Debug(m_transport,DebugGoOn,"Sctp conversion failed");
+		Debug(m_transport,DebugCrit,"Sctp conversion failed");
 		return false;
 	    }
 	    len = s->recvMsg((void*)buf,m_headerLen,addr,stream,flags);
@@ -1369,7 +1352,7 @@ bool StreamReader::readData()
 	if (m_transport->transType() == Transport::Sctp) {
 	    SctpSocket* s = static_cast<SctpSocket*>(m_socket);
 	    if (!s) {
-		Debug(m_transport,DebugGoOn,"Sctp conversion failed");
+		Debug(m_transport,DebugCrit,"Sctp conversion failed");
 		return false;
 	    }
 	    len = s->recvMsg((void*)buf1,m_totalPacketLen,addr,stream,flags);
@@ -1548,7 +1531,7 @@ bool MessageReader::sendMSG(const DataBlock& header, const DataBlock& msg, int s
 	if (m_transport->transType() == Transport::Sctp) {
 	    SctpSocket* s = static_cast<SctpSocket*>(m_socket);
 	    if (!s) {
-		Debug(m_transport,DebugGoOn,"Sctp conversion failed");
+		Debug(m_transport,DebugCrit,"Sctp conversion failed");
 		break;
 	    }
 	    int flags = 0;
@@ -1617,7 +1600,7 @@ bool MessageReader::readData()
 	int flags = 0;
 	SctpSocket* s = static_cast<SctpSocket*>(m_socket);
 	if (!s) {
-	    Debug(m_transport,DebugGoOn,"Sctp conversion failed");
+	    Debug(m_transport,DebugCrit,"Sctp conversion failed");
 	    return false;
 	}
 	SocketAddr addr;
