@@ -35,17 +35,6 @@ namespace { // anonymous
 
 #define RADIUS_MAXLEN 4096
 
-enum
-{
-    RadChanStart,
-    RadChanStop,
-    RadCallAnswered,
-    RadCallAuth,
-    RadCallExecute,
-    RadCallRinging,
-    RadUserLogin
-};
-
 enum
 {
     NoError = 0,
@@ -363,11 +352,15 @@ class RadiusClient : public GenObject
 public:
     RadiusClient()
 	: m_socket(0), m_authPort(0), m_acctPort(0),
-	  m_timeout(2000), m_retries(2)
+	  m_timeout(2000), m_retries(2), m_cisco(s_cisco), m_quintum(s_quintum)
 	{ }
     virtual ~RadiusClient();
     inline const String& server() const
 	{ return m_server; }
+    inline bool addCisco() const
+	{ return m_cisco; }
+    inline bool addQuintum() const
+	{ return m_quintum; }
     bool setRadServer(const char* host, int authport, int acctport, const char* secret, int timeoutms = 4000, int retries = 2);
     bool setRadServer(const NamedList& sect);
     bool addSocket();
@@ -395,6 +388,7 @@ private:
     String m_server,m_secret,m_section;
     unsigned int m_authPort,m_acctPort;
     int m_timeout, m_retries;
+    bool m_cisco, m_quintum;
     DataBlock m_authdata;
 };
 
@@ -410,6 +404,8 @@ protected:
 
 INIT_PLUGIN(RadiusModule);
 
+static const String s_fmtCisco("cisco_format");
+static const String s_fmtQuintum("quintum_format");
 
 class AuthHandler : public MessageHandler
 {
@@ -586,7 +582,7 @@ bool RadAttrib::decode(void* buf, unsigned int len, ObjList& list)
 	}
 	if (attr->isVendor()) {
 	    unsigned int len2 = attr->data().length();
-	    char* ptr = (char*)attr->data().data();
+	    unsigned char* ptr = (unsigned char*)attr->data().data();
 	    if ((len2 < 4) || !ptr) {
 		DDebug(&__plugin,DebugMild,"Invalid vendor attribute %u len=%u",attr->code(),len2);
 		attr->destruct();
@@ -637,7 +633,7 @@ RadAttrib::RadAttrib(const char* name, const char* value)
 	return;
     m_type = find(name,&m_vendor);
     if (!m_type) {
-	Debug(&__plugin,DebugGoOn,"Failed to find item %s in dictionary",name);
+	Debug(&__plugin,DebugWarn,"Failed to find item %s in dictionary",name);
 	return;
     }
     assign(value);
@@ -651,7 +647,7 @@ RadAttrib::RadAttrib(const char* name, int value)
 	return;
     m_type = find(name,&m_vendor);
     if (!m_type) {
-	Debug(&__plugin,DebugGoOn,"Failed to find item %s in dictionary",name);
+	Debug(&__plugin,DebugWarn,"Failed to find item %s in dictionary",name);
 	return;
     }
     assign(value);
@@ -664,7 +660,7 @@ RadAttrib::RadAttrib(const char* name, unsigned char subType, const char* value)
 	return;
     m_type = find(name,&m_vendor);
     if (!m_type) {
-	Debug(&__plugin,DebugGoOn,"Failed to find item %s in dictionary",name);
+	Debug(&__plugin,DebugWarn,"Failed to find item %s in dictionary",name);
 	return;
     }
     assign(subType,value);
@@ -703,7 +699,7 @@ bool RadAttrib::assign(const char* value)
 	    }
 	    break;
 	default:
-	    Debug(&__plugin,DebugGoOn,"Ignoring unknown attribute of type %d",m_type->type);
+	    Debug(&__plugin,DebugWarn,"Ignoring unknown attribute of type %d",m_type->type);
 	    return false;
     }
     return true;
@@ -733,7 +729,7 @@ bool RadAttrib::assign(int value)
 	    }
 	    break;
 	default:
-	    Debug(&__plugin,DebugGoOn,"Ignoring unknown attribute of type %d",m_type->type);
+	    Debug(&__plugin,DebugWarn,"Ignoring unknown attribute of type %d",m_type->type);
 	    return false;
     }
     return true;
@@ -969,7 +965,7 @@ int RadiusClient::makeRequest(int port, unsigned char request, unsigned char* re
     }
     int datalen = 20 + attrdata.length();
     if (datalen > RADIUS_MAXLEN) {
-	Debug(&__plugin,DebugGoOn,"Packet of %u bytes exceeds RADIUS maximum",datalen);
+	Debug(&__plugin,DebugWarn,"Packet of %u bytes exceeds RADIUS maximum %u",datalen,RADIUS_MAXLEN);
 	return UnknownErr;
     }
 
@@ -1013,7 +1009,7 @@ int RadiusClient::makeRequest(int port, unsigned char request, unsigned char* re
     // we have the data ready, send it and wait for an answer
     for (int r = m_retries; r > 0; r--) {
 	if (socket()->sendTo(radpckt.data(),radpckt.length(),sockAddr) == Socket::socketError()) {
-	    Alarm(&__plugin,"socket",DebugGoOn,"Packet sending error %d to %s:%d",
+	    Alarm(&__plugin,"socket",DebugCrit,"Packet sending error %d to %s:%d",
 		socket()->error(),sockAddr.host().c_str(),sockAddr.port());
 		return UnknownErr;
 	}
@@ -1180,8 +1176,7 @@ void RadiusClient::addAttributes(NamedList& params, NamedList* list)
 	NamedString* s = list->getParam(i);
 	if (!s)
 	    continue;
-	if (s->name().startsWith("inc:",false)) {
-	    String attr = s->name().substr(4).trimBlanks();
+	if ((s->name() == YSTRING("rad_include")) || s->name().startsWith("inc:",false)) {
 	    if (*s == *list)
 		Debug(&__plugin,DebugWarn,"Section '%s' includes itself!",s->c_str());
 	    else
@@ -1269,6 +1264,9 @@ bool RadiusClient::prepareAttributes(NamedList& params, bool forAcct, String* us
 	    NamedString* pair = sect->getParam(i2);
 	    if (!pair || pair->null())
 		continue;
+	    // ignore format control keys
+	    if ((pair->name() == s_fmtCisco) || (pair->name() == s_fmtQuintum))
+		continue;
 	    // ignore keys like rad_SOMETHING or SOMETHING:SOMETHING
 	    if (pair->name().startsWith("rad_",false) ||
 		(pair->name().find(':') >= 0))
@@ -1318,6 +1316,8 @@ bool RadiusClient::prepareAttributes(NamedList& params, bool forAcct, String* us
 
     Debug(&__plugin,DebugInfo,"Using sections [%s] and [%s] for %s",
 	m_section.c_str(),servName.c_str(),forAcct ? "accounting" : "authentication");
+    m_cisco = nasSect->getBoolValue(s_fmtCisco,servSect->getBoolValue(s_fmtCisco,s_cisco));
+    m_quintum = nasSect->getBoolValue(s_fmtQuintum,servSect->getBoolValue(s_fmtQuintum,s_quintum));
     addAttribute("User-Name",username);
     addAttribute("Calling-Station-Id",caller);
     addAttribute("Called-Station-Id",called);
@@ -1383,6 +1383,8 @@ bool AuthHandler::received(Message& msg)
     String user;
     if (!radclient.prepareAttributes(msg,false,&user))
 	return false;
+    bool cisco = msg.getBoolValue(s_fmtCisco,radclient.addCisco());
+    bool quintum = msg.getBoolValue(s_fmtQuintum,radclient.addQuintum());
     // TODO: process plaintext password
     if ((proto == "digest") || (proto == "sip")) {
 	const char* resp = msg.getValue("response");
@@ -1412,24 +1414,34 @@ bool AuthHandler::received(Message& msg)
     int sep = address.find(':');
     if (sep >= 0)
 	address = address.substr(0,sep);
-    if (s_cisco)
+    if (cisco)
 	radclient.addAttribute("h323-remote-address",address);
-    if (s_quintum)
+    if (quintum)
 	radclient.addAttribute("Quintum-h323-remote-address",address);
-    String billid = msg.getValue("billid");
-    if (billid) {
-	// create a Cisco-compatible conference ID
-	MD5 cid(billid);
-	String confid;
-	confid << cid.hexDigest().substr(0,8) << " ";
-	confid << cid.hexDigest().substr(8,8) << " ";
-	confid << cid.hexDigest().substr(16,8) << " ";
-	confid << cid.hexDigest().substr(24,8);
-	confid.toUpper();
-	if (s_cisco)
-	    radclient.addAttribute("h323-conf-id",confid);
-	if (s_quintum)
-	    radclient.addAttribute("Quintum-h323-conf-id",confid);
+    if (cisco || quintum) {
+	const String& billid = msg[YSTRING("billid")];
+	if (billid) {
+	    // create a Cisco-compatible conference ID
+	    MD5 cid(billid);
+	    String confid;
+	    confid << cid.hexDigest().substr(0,8) << " ";
+	    confid << cid.hexDigest().substr(8,8) << " ";
+	    confid << cid.hexDigest().substr(16,8) << " ";
+	    confid << cid.hexDigest().substr(24,8);
+	    confid.toUpper();
+	    String tmp("call-id=");
+	    if (address.null())
+		address = s_localAddr.host();
+	    tmp << billid << "@" << address;
+	    if (cisco) {
+		radclient.addAttribute("h323-conf-id",confid);
+		radclient.addAttribute("Cisco-AVPair",tmp);
+	    }
+	    if (quintum) {
+		radclient.addAttribute("Quintum-h323-conf-id",confid);
+		radclient.addAttribute("Quintum-AVPair",tmp);
+	    }
+	}
     }
 
     ObjList result;
@@ -1479,15 +1491,15 @@ static bool ciscoTime(double t, String& ret)
 
 bool AcctHandler::received(Message& msg)
 {
-    if (!msg.getBoolValue("cdrwrite_radius",true))
+    if (!msg.getBoolValue(YSTRING("cdrwrite_radius"),true))
 	return false;
-    String op = msg.getValue("operation");
     int acctStat = 0;
-    if (op == "initialize")
+    const String& op = msg[YSTRING("operation")];
+    if (op == YSTRING("initialize"))
 	acctStat = Acct_Start;
-    else if (op == "finalize")
+    else if (op == YSTRING("finalize"))
 	acctStat = Acct_Stop;
-    else if (op == "status")
+    else if ((op == YSTRING("status")) || (op == YSTRING("update")))
 	acctStat = Acct_Alive;
     else
 	return false;
@@ -1512,6 +1524,8 @@ bool AcctHandler::received(Message& msg)
     RadiusClient radclient;
     if (!radclient.prepareAttributes(msg))
 	return false;
+    bool cisco = msg.getBoolValue(s_fmtCisco,radclient.addCisco());
+    bool quintum = msg.getBoolValue(s_fmtQuintum,radclient.addQuintum());
 
     // create a Cisco-compatible conference ID
     MD5 cid(billid);
@@ -1528,12 +1542,12 @@ bool AcctHandler::received(Message& msg)
 
     radclient.addAttribute("Acct-Session-Id",sid.hexDigest());
     radclient.addAttribute("Acct-Status-Type",acctStat);
-    if (s_cisco) {
+    if (cisco) {
 	radclient.addAttribute("h323-call-origin",dir);
 	radclient.addAttribute("h323-conf-id",confid);
 	radclient.addAttribute("h323-remote-address",address);
     }
-    if (s_quintum) {
+    if (quintum) {
 	radclient.addAttribute("Quintum-h323-call-origin",dir);
 	radclient.addAttribute("Quintum-h323-conf-id",confid);
 	radclient.addAttribute("Quintum-h323-remote-address",address);
@@ -1543,51 +1557,50 @@ bool AcctHandler::received(Message& msg)
     if (address.null())
 	address = s_localAddr.host();
     tmp << billid << "@" << address;
-    if (s_cisco)
+    if (cisco)
 	radclient.addAttribute("Cisco-AVPair",tmp);
-    if (s_quintum)
+    if (quintum)
 	radclient.addAttribute("Quintum-AVPair",tmp);
 
     double t = msg.getDoubleValue("time");
-    ciscoTime(t,tmp);
-    if (s_cisco)
-	radclient.addAttribute("h323-setup-time",tmp);
-    if (s_quintum)
-	radclient.addAttribute("Quintum-h323-setup-time",tmp);
-    if (acctStat != Acct_Start) {
-	double d = msg.getDoubleValue("duration",-1);
-	if (d >= 0.0) {
-	    double d1 = msg.getDoubleValue("billtime");
-	    if (d1 > 0.0) {
-		ciscoTime(t+d-d1,tmp);
-		if (s_cisco)
-		    radclient.addAttribute("h323-connect-time",tmp);
-		if (s_quintum)
-		    radclient.addAttribute("Quintum-h323-connect-time",tmp);
-	    }
+    if (cisco || quintum) {
+	ciscoTime(t,tmp);
+	if (cisco)
+	    radclient.addAttribute("h323-setup-time",tmp);
+	if (quintum)
+	    radclient.addAttribute("Quintum-h323-setup-time",tmp);
+    }
+    double duration = msg.getDoubleValue("duration",-1);
+    double billtime = msg.getDoubleValue("billtime");
+    if ((cisco || quintum) && (Acct_Start != acctStat) && (duration >= 0.0)) {
+	if (billtime > 0.0) {
+	    ciscoTime(t+duration-billtime,tmp);
+	    if (cisco)
+		radclient.addAttribute("h323-connect-time",tmp);
+	    if (quintum)
+		radclient.addAttribute("Quintum-h323-connect-time",tmp);
 	}
     }
 
-    if (acctStat == Acct_Stop) {
-	double d = msg.getDoubleValue("duration",-1);
-	if (d >= 0.0) {
-	    ciscoTime(t+d,tmp);
-	    if (s_cisco)
+    if (Acct_Stop == acctStat) {
+	if ((cisco || quintum) && (duration >= 0.0)) {
+	    ciscoTime(t+duration,tmp);
+	    if (cisco)
 		radclient.addAttribute("h323-disconnect-time",tmp);
-	    if (s_quintum)
+	    if (quintum)
 		radclient.addAttribute("Quintum-h323-disconnect-time",tmp);
 	}
 
-	radclient.addAttribute("Acct-Session-Time",(int)msg.getDoubleValue("billtime"));
+	radclient.addAttribute("Acct-Session-Time",(int)billtime);
 	int cause = lookup(msg.getValue("status"),dict_errors,-1,10);
 	if (cause >= 0)
 	    radclient.addAttribute("Acct-Terminate-Cause",cause);
 	String tmp = msg.getValue("reason");
 	if (tmp) {
 	    tmp = "disconnect-text=" + tmp;
-	    if (s_cisco)
+	    if (cisco)
 		radclient.addAttribute("Cisco-AVPair",tmp);
-	    if (s_quintum)
+	    if (quintum)
 		radclient.addAttribute("Quintum-AVPair",tmp);
 	}
     }
@@ -1600,23 +1613,22 @@ bool RadiusHandler::received(Message& msg)
     bool auth = msg.getBoolValue(YSTRING("auth"),true);
     int acctStat = 0;
     if (!auth) {
-	String op = msg.getValue("operation");
-	if (op == "initialize")
+	const String& op = msg[YSTRING("operation")];
+	if (op == YSTRING("initialize"))
 	    acctStat = Acct_Start;
-	else if (op == "finalize")
+	else if (op == YSTRING("finalize"))
 	    acctStat = Acct_Stop;
-	else if (op == "status") {
+	else if ((op == YSTRING("status")) || (op == YSTRING("update")))
 	    acctStat = Acct_Alive;
-	} else
+	else
 	    return false;
     }
     RadiusClient radclient;
     if (!radclient.prepareAttributes(msg))
 	return false;
 
-    if (!auth) {
+    if (!auth)
 	radclient.addAttribute("Acct-Status-Type",acctStat);
-    }
 
     ObjList result;
     if (auth && radclient.doAuthenticate(&result) != AuthSuccess) {
@@ -1655,8 +1667,8 @@ void RadiusModule::initialize()
     s_pb_enabled = s_cfg.getBoolValue("portabill","enabled",false);
     s_pb_parallel = s_cfg.getBoolValue("portabill","parallel",false);
     s_pb_simplify = s_cfg.getBoolValue("portabill","simplify",false);
-    s_cisco = s_cfg.getBoolValue("general","cisco_format",true);
-    s_quintum = s_cfg.getBoolValue("general","quintum_format",true);
+    s_cisco = s_cfg.getBoolValue("general",s_fmtCisco,true);
+    s_quintum = s_cfg.getBoolValue("general",s_fmtQuintum,true);
     s_pb_stoperror = s_cfg.getValue("portabill","stoperror","busy");
     s_pb_maxcall = s_cfg.getValue("portabill","maxcall");
     s_cfgMutex.unlock();
@@ -1680,11 +1692,11 @@ void RadiusModule::initialize()
 
     // we only have UDP support
     if (!s_localSock.create(PF_INET,SOCK_DGRAM,IPPROTO_IP)) {
-	Alarm(this,"socket",DebugGoOn,"Error creating socket. Radius functions unavailable");
+	Alarm(this,"socket",DebugCrit,"Error creating socket. Radius functions unavailable");
 	return;
     }
     if (!s_localSock.bind(s_localAddr)) {
-	Alarm(this,"socket",DebugWarn,"Error %d binding to %s:%d. Radius functions unavailable",
+	Alarm(this,"socket",DebugCrit,"Error %d binding to %s:%d. Radius functions unavailable",
 	    s_localSock.error(),s_localAddr.host().c_str(),s_localAddr.port());
 	return;
     }
