@@ -46,7 +46,7 @@ static const TokenDict dict_payloads[] = {
     {"adpcm",  IAXFormat::ADPCM},
     {"g726",   IAXFormat::G726},
     {"slin",   IAXFormat::SLIN},
-    {"g722",   IAXFormat::G722},
+    {"g722/16000", IAXFormat::G722},
     {"amr",    IAXFormat::AMR},
     {"gsmhr",  IAXFormat::GSM_HR},
     {0, 0}
@@ -2492,8 +2492,10 @@ YIAXConnection::YIAXConnection(IAXTransaction* tr, Message* msg, NamedList* para
     Debug(this,DebugAll,"%s call. Transaction (%p) callno=%u [%p]",
 	isOutgoing() ? "Outgoing" : "Incoming",tr,tr->localCallNo(),this);
     setMaxcall(msg);
-    if (msg)
+    if (msg) {
 	setMaxPDD(*msg);
+	setChanParams(*msg);
+    }
     if (tr)
 	m_address << tr->remoteAddr().host() << ":" << tr->remoteAddr().port();
     if (msg)
