@@ -42,7 +42,7 @@ static TokenDict dict_payloads[] = {
     { "lpc10",         7 },
     { "slin",         11 },
     { "g726",          2 },
-    { "g722",          9 },
+    { "g722/16000",    9 },
     { "g723",          4 },
     { "g728",         15 },
     { "g729",         18 },
@@ -63,9 +63,14 @@ static TokenDict dict_payloads[] = {
     { "mjpeg",        26 },
     { "h261",         31 },
     { "h263",         34 },
+    { "h263-1998",   111 },
+    { "h263-2000",   112 },
+    { "h264",        114 },
+    { "vp8",         113 },
+    { "vp9",         115 },
     { "mpv",          32 },
     { "mp2t",         33 },
-    { "mp4v",         98 },
+    { "mp4v",        110 },
     { 0 ,              0 },
 };
 
@@ -843,7 +848,7 @@ bool YRTPWrapper::startSRTP(const String& suite, const String& keyParams, const
     RTPSecure* srtp = new RTPSecure;
     if (srtp->supported(m_rtp) && srtp->setup(suite,keyParams,paramList)) {
 	m_rtp->receiver()->security(srtp);
-	Debug(&splugin,DebugNote,"Started SRTP suite '%s' [%p]",suite.c_str(),this);
+	Debug(&splugin,DebugCall,"Started SRTP suite '%s' [%p]",suite.c_str(),this);
 	return true;
     }
     TelEngine::destruct(srtp);
